# Implementierung von IStringCalculator
from abc import ABC, abstractmethod

# Definition des Interfaces für den StringCalculator
class IStringCalculator(ABC):
    @abstractmethod
    def add(self, numbers: str) -> int:
        pass




class StringCalculator(IStringCalculator):
    def add(self, numbers: str) -> int:
        if not numbers:
            return 0

        delimiter = ","


        if numbers.startswith("//"):
            header, numbers = numbers.split("\n", 1)

            # Mehrzeichen-Delimiter: z. B. //[***]
            if header.startswith("//[") and header.endswith("]"):
                delimiter = header[3:-1]  # Extrahiere z. B. "***"
            else:
                delimiter = header[2:]  # Einzeichen-Delimiter

        # Ersetze \n durch Komma (Standard)
        numbers = numbers.replace("\n", ",")

        # Delimiter ersetzen durch Komma (evtl. mehrere Zeichen)
        numbers = numbers.replace(delimiter, ",")

        number_list = numbers.split(",")

        # Ignoriere leere oder whitespace-only Einträge
        cleaned_numbers = [
            int(n.strip()) for n in number_list if n.strip() != ""
        ]

        # Negativ-Prüfung
        negatives = [n for n in cleaned_numbers if n < 0]
        if negatives:
            raise ValueError(f"negatives not allowed: {', '.join(map(str, negatives))}")

        # Zahlen > 1000 ignorieren
        cleaned_numbers = [n for n in cleaned_numbers if n <= 1000]

        return sum(cleaned_numbers)
    
import unittest

class TestStringCalculator(unittest.TestCase):
    def setUp(self):
        # Vor jedem Test wird eine Instanz des Calculators erstellt
        self.calc = StringCalculator()

    def test_add_empty_string(self):
        # Leerer String → 0
        self.assertEqual(self.calc.add(""), 0)

    def test_add_single_number(self):
        # Ein einzelner Wert → int
        self.assertEqual(self.calc.add("1"), 1)

    def test_add_two_numbers(self):
        # Zwei Zahlen → Summe
        self.assertEqual(self.calc.add("1,2"), 3)

    def test_add_multiple_numbers(self):
        # Mehr als zwei Zahlen → Summe
        self.assertEqual(self.calc.add("1,2,3"), 6)
        self.assertEqual(self.calc.add("1,2,3,4,5"), 15)

    def test_add_ignores_empty_entries(self):
        # Leere Einträge ignorieren → Summe
        self.assertEqual(self.calc.add("1,,2"), 3)
        self.assertEqual(self.calc.add("1, ,2"), 3)
        self.assertEqual(self.calc.add(",,1,,2,,,"), 3)

    def test_add_with_newlines_between_numbers(self):
        # Zeilenumbrüche als Trennzeichen erlauben → Summe
        self.assertEqual(self.calc.add("1\n2,3"), 6)
        self.assertEqual(self.calc.add("1,2\n3"), 6)
        self.assertEqual(self.calc.add("1\n2\n3"), 6)

    def test_add_raises_exception_on_negative_number(self):
        # Negative Zahlen → Exception mit allen negativen Werten
        with self.assertRaises(ValueError) as context:
            self.calc.add("1,-2")
        self.assertIn("negatives not allowed: -2", str(context.exception))

    def test_add_raises_exception_on_multiple_negatives(self):
        # Negative Zahlen → Exception mit allen negativen Werten
        with self.assertRaises(ValueError) as context:
            self.calc.add("-1,-2,3")
        self.assertIn("negatives not allowed: -1, -2", str(context.exception))

    def test_add_with_custom_delimiter(self):
        # Benutzerdefinierter Delimiter über Präfix: "//<delimiter>\n"
        self.assertEqual(self.calc.add("//;\n1;2"), 3)
        self.assertEqual(self.calc.add("//|\n4|5|6"), 15)
        self.assertEqual(self.calc.add("//_\n7_8_9"), 24)

    def test_add_ignores_numbers_greater_than_1000(self):
        # Zahlen größer als 1000 ignorieren
        self.assertEqual(self.calc.add("2,1001"), 2)
        self.assertEqual(self.calc.add("1000,1"), 1001)
        self.assertEqual(self.calc.add("1234,1001,3"), 3)

    def test_add_with_multi_char_delimiter(self):
        # Delimiter mit beliebiger Länge (Format: //[delimiter]\n)
        self.assertEqual(self.calc.add("//[***]\n1***2***3"), 6)
        self.assertEqual(self.calc.add("//[abc]\n4abc5abc6"), 15)
        self.assertEqual(self.calc.add("//[+]\n1+2+3"), 6)

if __name__ == "__main__":
    unittest.main()